package cn.com.infosec.jce.provider;

import cn.com.infosec.asn1.ASN1OctetString;
import cn.com.infosec.asn1.ASN1Sequence;
import cn.com.infosec.asn1.DERIA5String;
import cn.com.infosec.asn1.x509.GeneralName;
import cn.com.infosec.asn1.x509.GeneralSubtree;
import cn.com.infosec.util.Arrays;
import cn.com.infosec.util.Strings;
import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

public class PKIXNameConstraintValidator {
   private Set excludedSubtreesDN;
   private Set excludedSubtreesDNS;
   private Set excludedSubtreesEmail;
   private Set excludedSubtreesURI;
   private Set excludedSubtreesIP;
   private Set permittedSubtreesDN;
   private Set permittedSubtreesDNS;
   private Set permittedSubtreesEmail;
   private Set permittedSubtreesURI;
   private Set permittedSubtreesIP;

   public PKIXNameConstraintValidator() {
      PKIXNameConstraintValidator var10000 = this;
      PKIXNameConstraintValidator var10001 = this;
      PKIXNameConstraintValidator var10002 = this;
      PKIXNameConstraintValidator var10003 = this;
      PKIXNameConstraintValidator var10004 = this;
      super();
      HashSet var1;
      var1 = new HashSet.<init>();
      var10004.excludedSubtreesDN = var1;
      var1 = new HashSet.<init>();
      var10003.excludedSubtreesDNS = var1;
      var1 = new HashSet.<init>();
      var10002.excludedSubtreesEmail = var1;
      var1 = new HashSet.<init>();
      var10001.excludedSubtreesURI = var1;
      var1 = new HashSet.<init>();
      var10000.excludedSubtreesIP = var1;
   }

   private static boolean withinDNSubtree(ASN1Sequence dns, ASN1Sequence subtree) {
      if (subtree.size() < 1) {
         return false;
      } else if (subtree.size() > dns.size()) {
         return false;
      } else {
         for(int var2 = subtree.size() - 1; var2 >= 0; --var2) {
            if (!subtree.getObjectAt(var2).equals(dns.getObjectAt(var2))) {
               return false;
            }
         }

         return true;
      }
   }

   private void checkPermittedDN(Set permitted, ASN1Sequence dns) {
      if (permitted != null) {
         if (!permitted.isEmpty() || dns.size() != 0) {
            Iterator var3 = permitted.iterator();

            do {
               if (!var3.hasNext()) {
                  throw new PKIXNameConstraintValidatorException("Subject distinguished name is not from a permitted subtree");
               }
            } while(!withinDNSubtree(dns, (ASN1Sequence)var3.next()));

         }
      }
   }

   private void checkExcludedDN(Set excluded, ASN1Sequence dns) {
      if (!excluded.isEmpty()) {
         Iterator var3 = excluded.iterator();

         while(var3.hasNext()) {
            if (withinDNSubtree(dns, (ASN1Sequence)var3.next())) {
               throw new PKIXNameConstraintValidatorException("Subject distinguished name is from an excluded subtree");
            }
         }

      }
   }

   private Set intersectDN(Set permitted, Set dns) {
      HashSet var6;
      var6 = new HashSet.<init>();
      Iterator dns1 = dns.iterator();

      while(true) {
         while(dns1.hasNext()) {
            ASN1Sequence var3 = ASN1Sequence.getInstance(((GeneralSubtree)dns1.next()).getBase().getName().getDERObject());
            if (permitted == null) {
               if (var3 != null) {
                  var6.add(var3);
               }
            } else {
               Iterator var4 = permitted.iterator();

               while(var4.hasNext()) {
                  ASN1Sequence var5;
                  if (withinDNSubtree(var3, var5 = (ASN1Sequence)var4.next())) {
                     var6.add(var3);
                  } else if (withinDNSubtree(var5, var3)) {
                     var6.add(var5);
                  }
               }
            }
         }

         return var6;
      }
   }

   private Set unionDN(Set excluded, ASN1Sequence dn) {
      if (excluded.isEmpty()) {
         if (dn == null) {
            return excluded;
         } else {
            excluded.add(dn);
            return excluded;
         }
      } else {
         HashSet var4;
         var4 = new HashSet.<init>();
         Iterator excluded1 = excluded.iterator();

         while(excluded1.hasNext()) {
            ASN1Sequence var3;
            if (withinDNSubtree(dn, var3 = (ASN1Sequence)excluded1.next())) {
               var4.add(var3);
            } else if (withinDNSubtree(var3, dn)) {
               var4.add(dn);
            } else {
               var4.add(var3);
               var4.add(dn);
            }
         }

         return var4;
      }
   }

   private Set intersectEmail(Set permitted, Set emails) {
      Set var10000 = emails;
      HashSet emails1;
      emails1 = new HashSet.<init>();
      Iterator var3 = var10000.iterator();

      while(true) {
         while(var3.hasNext()) {
            String var4 = this.extractNameAsString(((GeneralSubtree)var3.next()).getBase());
            if (permitted == null) {
               if (var4 != null) {
                  emails1.add(var4);
               }
            } else {
               Iterator var5 = permitted.iterator();

               while(var5.hasNext()) {
                  this.intersectEmail(var4, (String)var5.next(), emails1);
               }
            }
         }

         return emails1;
      }
   }

   private Set unionEmail(Set excluded, String email) {
      if (excluded.isEmpty()) {
         if (email == null) {
            return excluded;
         } else {
            excluded.add(email);
            return excluded;
         }
      } else {
         Set var10000 = excluded;
         HashSet excluded1;
         excluded1 = new HashSet.<init>();
         Iterator var3 = var10000.iterator();

         while(var3.hasNext()) {
            this.unionEmail((String)var3.next(), email, excluded1);
         }

         return excluded1;
      }
   }

   private Set intersectIP(Set permitted, Set ips) {
      Set var10000 = ips;
      HashSet ips1;
      ips1 = new HashSet.<init>();
      Iterator var3 = var10000.iterator();

      while(true) {
         while(var3.hasNext()) {
            byte[] var4 = ASN1OctetString.getInstance(((GeneralSubtree)var3.next()).getBase().getName()).getOctets();
            if (permitted == null) {
               if (var4 != null) {
                  ips1.add(var4);
               }
            } else {
               Iterator var5 = permitted.iterator();

               while(var5.hasNext()) {
                  ips1.addAll(this.intersectIPRange((byte[])var5.next(), var4));
               }
            }
         }

         return ips1;
      }
   }

   private Set unionIP(Set excluded, byte[] ip) {
      if (excluded.isEmpty()) {
         if (ip == null) {
            return excluded;
         } else {
            excluded.add(ip);
            return excluded;
         }
      } else {
         Set var10000 = excluded;
         HashSet excluded1;
         excluded1 = new HashSet.<init>();
         Iterator var3 = var10000.iterator();

         while(var3.hasNext()) {
            excluded1.addAll(this.unionIPRange((byte[])var3.next(), ip));
         }

         return excluded1;
      }
   }

   private Set unionIPRange(byte[] ipWithSubmask1, byte[] ipWithSubmask2) {
      HashSet var3;
      var3 = new HashSet.<init>();
      if (Arrays.areEqual(ipWithSubmask1, ipWithSubmask2)) {
         var3.add(ipWithSubmask1);
      } else {
         var3.add(ipWithSubmask1);
         var3.add(ipWithSubmask2);
      }

      return var3;
   }

   private Set intersectIPRange(byte[] ipWithSubmask1, byte[] ipWithSubmask2) {
      if (ipWithSubmask1.length != ipWithSubmask2.length) {
         return Collections.EMPTY_SET;
      } else {
         byte[][] var10001 = this.extractIPsAndSubnetMasks(ipWithSubmask1, ipWithSubmask2);
         ipWithSubmask1 = var10001[0];
         ipWithSubmask2 = var10001[1];
         byte[] var3 = var10001[2];
         byte[] var4 = var10001[3];
         byte[][] ipWithSubmask11;
         byte[][] var10000 = ipWithSubmask11 = this.minMaxIPs(ipWithSubmask1, ipWithSubmask2, var3, var4);
         var3 = min(var10000[1], ipWithSubmask11[3]);
         if (compareTo(max(var10000[0], ipWithSubmask11[2]), var3) == 1) {
            return Collections.EMPTY_SET;
         } else {
            PKIXNameConstraintValidator var7 = this;
            byte[] var5 = or(ipWithSubmask11[0], ipWithSubmask11[2]);
            return Collections.singleton(var7.ipWithSubnetMask(var5, or(ipWithSubmask2, var4)));
         }
      }
   }

   private byte[] ipWithSubnetMask(byte[] ip, byte[] subnetMask) {
      byte[] var3;
      int var4;
      byte[] var10000 = var3 = new byte[(var4 = ip.length) * 2];
      System.arraycopy(ip, 0, var3, 0, var4);
      System.arraycopy(subnetMask, 0, var3, var4, var4);
      return var10000;
   }

   private byte[][] extractIPsAndSubnetMasks(byte[] ipWithSubmask1, byte[] ipWithSubmask2) {
      byte[] var10000 = ipWithSubmask2;
      byte[] var10001 = ipWithSubmask2;
      int var5;
      int var10002 = var5 = ipWithSubmask1.length / 2;
      byte[] var10004 = ipWithSubmask1;
      byte[] var10006 = ipWithSubmask1;
      ipWithSubmask1 = new byte[var5];
      ipWithSubmask2 = new byte[var5];
      System.arraycopy(var10006, 0, ipWithSubmask1, 0, var5);
      System.arraycopy(var10004, var5, ipWithSubmask2, 0, var5);
      byte[] var3 = new byte[var10002];
      byte[] var4 = new byte[var10002];
      System.arraycopy(var10001, 0, var3, 0, var5);
      System.arraycopy(var10000, var5, var4, 0, var5);
      return new byte[][]{ipWithSubmask1, ipWithSubmask2, var3, var4};
   }

   private byte[][] minMaxIPs(byte[] ip1, byte[] subnetmask1, byte[] ip2, byte[] subnetmask2) {
      int var10000 = ip1.length;
      int var10001 = ip1.length;
      int var10;
      int var10002 = var10 = ip1.length;
      byte[] var5 = new byte[var10001];
      byte[] var6 = new byte[var10002];
      byte[] var7 = new byte[var10001];
      byte[] var8 = new byte[var10000];

      for(int var9 = 0; var9 < var10; ++var9) {
         var5[var9] = (byte)(ip1[var9] & subnetmask1[var9]);
         var6[var9] = (byte)(ip1[var9] & subnetmask1[var9] | ~subnetmask1[var9]);
         var7[var9] = (byte)(ip2[var9] & subnetmask2[var9]);
         var8[var9] = (byte)(ip2[var9] & subnetmask2[var9] | ~subnetmask2[var9]);
      }

      return new byte[][]{var5, var6, var7, var8};
   }

   private void checkPermittedEmail(Set permitted, String email) {
      if (permitted != null) {
         Iterator var3 = permitted.iterator();

         while(var3.hasNext()) {
            if (this.emailIsConstrained(email, (String)var3.next())) {
               return;
            }
         }

         if (email.length() != 0 || permitted.size() != 0) {
            throw new PKIXNameConstraintValidatorException("Subject email address is not from a permitted subtree.");
         }
      }
   }

   private void checkExcludedEmail(Set excluded, String email) {
      if (!excluded.isEmpty()) {
         Iterator excluded1 = excluded.iterator();

         while(excluded1.hasNext()) {
            if (this.emailIsConstrained(email, (String)excluded1.next())) {
               throw new PKIXNameConstraintValidatorException("Email address is from an excluded subtree.");
            }
         }

      }
   }

   private void checkPermittedIP(Set permitted, byte[] ip) {
      if (permitted != null) {
         Iterator var3 = permitted.iterator();

         while(var3.hasNext()) {
            if (this.isIPConstrained(ip, (byte[])var3.next())) {
               return;
            }
         }

         if (ip.length != 0 || permitted.size() != 0) {
            throw new PKIXNameConstraintValidatorException("IP is not from a permitted subtree.");
         }
      }
   }

   private void checkExcludedIP(Set excluded, byte[] ip) {
      if (!excluded.isEmpty()) {
         Iterator excluded1 = excluded.iterator();

         while(excluded1.hasNext()) {
            if (this.isIPConstrained(ip, (byte[])excluded1.next())) {
               throw new PKIXNameConstraintValidatorException("IP is from an excluded subtree.");
            }
         }

      }
   }

   private boolean isIPConstrained(byte[] ip, byte[] constraint) {
      int var7;
      if ((var7 = ip.length) != constraint.length / 2) {
         return false;
      } else {
         byte[] var3 = new byte[var7];
         System.arraycopy(constraint, var7, var3, 0, var7);
         byte[] var4 = new byte[var7];
         byte[] var5 = new byte[var7];

         for(int var6 = 0; var6 < var7; ++var6) {
            var4[var6] = (byte)(constraint[var6] & var3[var6]);
            var5[var6] = (byte)(ip[var6] & var3[var6]);
         }

         return Arrays.areEqual(var4, var5);
      }
   }

   private boolean emailIsConstrained(String email, String constraint) {
      String var3 = email.substring(email.indexOf(64) + 1);
      if (constraint.indexOf(64) != -1) {
         if (email.equalsIgnoreCase(constraint)) {
            return true;
         }
      } else if (constraint.charAt(0) != '.') {
         if (var3.equalsIgnoreCase(constraint)) {
            return true;
         }
      } else if (this.withinDomain(var3, constraint)) {
         return true;
      }

      return false;
   }

   private boolean withinDomain(String testDomain, String domain) {
      if (domain.startsWith(".")) {
         domain = domain.substring(1);
      }

      String[] var4 = Strings.split(domain, '.');
      String[] testDomain1;
      if ((testDomain1 = Strings.split(testDomain, '.')).length <= var4.length) {
         return false;
      } else {
         int domain1 = testDomain1.length - var4.length;

         for(int var3 = -1; var3 < var4.length; ++var3) {
            if (var3 == -1) {
               if (testDomain1[var3 + domain1].equals("")) {
                  return false;
               }
            } else if (!var4[var3].equalsIgnoreCase(testDomain1[var3 + domain1])) {
               return false;
            }
         }

         return true;
      }
   }

   private void checkPermittedDNS(Set permitted, String dns) {
      if (permitted != null) {
         Iterator var3 = permitted.iterator();

         String var4;
         do {
            if (!var3.hasNext()) {
               if (dns.length() == 0 && permitted.size() == 0) {
                  return;
               }

               throw new PKIXNameConstraintValidatorException("DNS is not from a permitted subtree.");
            }
         } while(!this.withinDomain(dns, var4 = (String)var3.next()) && !dns.equalsIgnoreCase(var4));

      }
   }

   private void checkExcludedDNS(Set excluded, String dns) {
      if (!excluded.isEmpty()) {
         Iterator excluded1 = excluded.iterator();

         String var3;
         do {
            if (!excluded1.hasNext()) {
               return;
            }
         } while(!this.withinDomain(dns, var3 = (String)excluded1.next()) && !dns.equalsIgnoreCase(var3));

         throw new PKIXNameConstraintValidatorException("DNS is from an excluded subtree.");
      }
   }

   private void unionEmail(String email1, String email2, Set union) {
      if (email1.indexOf(64) != -1) {
         String var4 = email1.substring(email1.indexOf(64) + 1);
         if (email2.indexOf(64) != -1) {
            if (email1.equalsIgnoreCase(email2)) {
               union.add(email1);
            } else {
               union.add(email1);
               union.add(email2);
            }
         } else if (email2.startsWith(".")) {
            if (this.withinDomain(var4, email2)) {
               union.add(email2);
            } else {
               union.add(email1);
               union.add(email2);
            }
         } else if (var4.equalsIgnoreCase(email2)) {
            union.add(email2);
         } else {
            union.add(email1);
            union.add(email2);
         }
      } else if (email1.startsWith(".")) {
         if (email2.indexOf(64) != -1) {
            if (this.withinDomain(email2.substring(email1.indexOf(64) + 1), email1)) {
               union.add(email1);
            } else {
               union.add(email1);
               union.add(email2);
            }
         } else if (email2.startsWith(".")) {
            if (!this.withinDomain(email1, email2) && !email1.equalsIgnoreCase(email2)) {
               if (this.withinDomain(email2, email1)) {
                  union.add(email1);
               } else {
                  union.add(email1);
                  union.add(email2);
               }
            } else {
               union.add(email2);
            }
         } else if (this.withinDomain(email2, email1)) {
            union.add(email1);
         } else {
            union.add(email1);
            union.add(email2);
         }
      } else if (email2.indexOf(64) != -1) {
         if (email2.substring(email1.indexOf(64) + 1).equalsIgnoreCase(email1)) {
            union.add(email1);
         } else {
            union.add(email1);
            union.add(email2);
         }
      } else if (email2.startsWith(".")) {
         if (this.withinDomain(email1, email2)) {
            union.add(email2);
         } else {
            union.add(email1);
            union.add(email2);
         }
      } else if (email1.equalsIgnoreCase(email2)) {
         union.add(email1);
      } else {
         union.add(email1);
         union.add(email2);
      }

   }

   private void unionURI(String email1, String email2, Set union) {
      if (email1.indexOf(64) != -1) {
         String var4 = email1.substring(email1.indexOf(64) + 1);
         if (email2.indexOf(64) != -1) {
            if (email1.equalsIgnoreCase(email2)) {
               union.add(email1);
            } else {
               union.add(email1);
               union.add(email2);
            }
         } else if (email2.startsWith(".")) {
            if (this.withinDomain(var4, email2)) {
               union.add(email2);
            } else {
               union.add(email1);
               union.add(email2);
            }
         } else if (var4.equalsIgnoreCase(email2)) {
            union.add(email2);
         } else {
            union.add(email1);
            union.add(email2);
         }
      } else if (email1.startsWith(".")) {
         if (email2.indexOf(64) != -1) {
            if (this.withinDomain(email2.substring(email1.indexOf(64) + 1), email1)) {
               union.add(email1);
            } else {
               union.add(email1);
               union.add(email2);
            }
         } else if (email2.startsWith(".")) {
            if (!this.withinDomain(email1, email2) && !email1.equalsIgnoreCase(email2)) {
               if (this.withinDomain(email2, email1)) {
                  union.add(email1);
               } else {
                  union.add(email1);
                  union.add(email2);
               }
            } else {
               union.add(email2);
            }
         } else if (this.withinDomain(email2, email1)) {
            union.add(email1);
         } else {
            union.add(email1);
            union.add(email2);
         }
      } else if (email2.indexOf(64) != -1) {
         if (email2.substring(email1.indexOf(64) + 1).equalsIgnoreCase(email1)) {
            union.add(email1);
         } else {
            union.add(email1);
            union.add(email2);
         }
      } else if (email2.startsWith(".")) {
         if (this.withinDomain(email1, email2)) {
            union.add(email2);
         } else {
            union.add(email1);
            union.add(email2);
         }
      } else if (email1.equalsIgnoreCase(email2)) {
         union.add(email1);
      } else {
         union.add(email1);
         union.add(email2);
      }

   }

   private Set intersectDNS(Set permitted, Set dnss) {
      Set var10000 = dnss;
      HashSet dnss1;
      dnss1 = new HashSet.<init>();
      Iterator var3 = var10000.iterator();

      while(true) {
         while(var3.hasNext()) {
            String var4 = this.extractNameAsString(((GeneralSubtree)var3.next()).getBase());
            if (permitted == null) {
               if (var4 != null) {
                  dnss1.add(var4);
               }
            } else {
               Iterator var5 = permitted.iterator();

               while(var5.hasNext()) {
                  String var6;
                  if (this.withinDomain(var6 = (String)var5.next(), var4)) {
                     dnss1.add(var6);
                  } else if (this.withinDomain(var4, var6)) {
                     dnss1.add(var4);
                  }
               }
            }
         }

         return dnss1;
      }
   }

   private void intersectEmail(String email1, String email2, Set intersect) {
      if (email1.indexOf(64) != -1) {
         String var4 = email1.substring(email1.indexOf(64) + 1);
         if (email2.indexOf(64) != -1) {
            if (email1.equalsIgnoreCase(email2)) {
               intersect.add(email1);
            }
         } else if (email2.startsWith(".")) {
            if (this.withinDomain(var4, email2)) {
               intersect.add(email1);
            }
         } else if (var4.equalsIgnoreCase(email2)) {
            intersect.add(email1);
         }
      } else if (email1.startsWith(".")) {
         if (email2.indexOf(64) != -1) {
            if (this.withinDomain(email2.substring(email1.indexOf(64) + 1), email1)) {
               intersect.add(email2);
            }
         } else if (email2.startsWith(".")) {
            if (!this.withinDomain(email1, email2) && !email1.equalsIgnoreCase(email2)) {
               if (this.withinDomain(email2, email1)) {
                  intersect.add(email2);
               }
            } else {
               intersect.add(email1);
            }
         } else if (this.withinDomain(email2, email1)) {
            intersect.add(email2);
         }
      } else if (email2.indexOf(64) != -1) {
         if (email2.substring(email2.indexOf(64) + 1).equalsIgnoreCase(email1)) {
            intersect.add(email2);
         }
      } else if (email2.startsWith(".")) {
         if (this.withinDomain(email1, email2)) {
            intersect.add(email1);
         }
      } else if (email1.equalsIgnoreCase(email2)) {
         intersect.add(email1);
      }

   }

   private void checkExcludedURI(Set excluded, String uri) {
      if (!excluded.isEmpty()) {
         Iterator excluded1 = excluded.iterator();

         while(excluded1.hasNext()) {
            if (this.isUriConstrained(uri, (String)excluded1.next())) {
               throw new PKIXNameConstraintValidatorException("URI is from an excluded subtree.");
            }
         }

      }
   }

   private Set intersectURI(Set permitted, Set uris) {
      Set var10000 = uris;
      HashSet uris1;
      uris1 = new HashSet.<init>();
      Iterator var3 = var10000.iterator();

      while(true) {
         while(var3.hasNext()) {
            String var4 = this.extractNameAsString(((GeneralSubtree)var3.next()).getBase());
            if (permitted == null) {
               if (var4 != null) {
                  uris1.add(var4);
               }
            } else {
               Iterator var5 = permitted.iterator();

               while(var5.hasNext()) {
                  this.intersectURI((String)var5.next(), var4, uris1);
               }
            }
         }

         return uris1;
      }
   }

   private Set unionURI(Set excluded, String uri) {
      if (excluded.isEmpty()) {
         if (uri == null) {
            return excluded;
         } else {
            excluded.add(uri);
            return excluded;
         }
      } else {
         Set var10000 = excluded;
         HashSet excluded1;
         excluded1 = new HashSet.<init>();
         Iterator var3 = var10000.iterator();

         while(var3.hasNext()) {
            this.unionURI((String)var3.next(), uri, excluded1);
         }

         return excluded1;
      }
   }

   private void intersectURI(String email1, String email2, Set intersect) {
      if (email1.indexOf(64) != -1) {
         String var4 = email1.substring(email1.indexOf(64) + 1);
         if (email2.indexOf(64) != -1) {
            if (email1.equalsIgnoreCase(email2)) {
               intersect.add(email1);
            }
         } else if (email2.startsWith(".")) {
            if (this.withinDomain(var4, email2)) {
               intersect.add(email1);
            }
         } else if (var4.equalsIgnoreCase(email2)) {
            intersect.add(email1);
         }
      } else if (email1.startsWith(".")) {
         if (email2.indexOf(64) != -1) {
            if (this.withinDomain(email2.substring(email1.indexOf(64) + 1), email1)) {
               intersect.add(email2);
            }
         } else if (email2.startsWith(".")) {
            if (!this.withinDomain(email1, email2) && !email1.equalsIgnoreCase(email2)) {
               if (this.withinDomain(email2, email1)) {
                  intersect.add(email2);
               }
            } else {
               intersect.add(email1);
            }
         } else if (this.withinDomain(email2, email1)) {
            intersect.add(email2);
         }
      } else if (email2.indexOf(64) != -1) {
         if (email2.substring(email2.indexOf(64) + 1).equalsIgnoreCase(email1)) {
            intersect.add(email2);
         }
      } else if (email2.startsWith(".")) {
         if (this.withinDomain(email1, email2)) {
            intersect.add(email1);
         }
      } else if (email1.equalsIgnoreCase(email2)) {
         intersect.add(email1);
      }

   }

   private void checkPermittedURI(Set permitted, String uri) {
      if (permitted != null) {
         Iterator var3 = permitted.iterator();

         while(var3.hasNext()) {
            if (this.isUriConstrained(uri, (String)var3.next())) {
               return;
            }
         }

         if (uri.length() != 0 || permitted.size() != 0) {
            throw new PKIXNameConstraintValidatorException("URI is not from a permitted subtree.");
         }
      }
   }

   private boolean isUriConstrained(String uri, String constraint) {
      uri = extractHostFromURL(uri);
      if (!constraint.startsWith(".")) {
         if (uri.equalsIgnoreCase(constraint)) {
            return true;
         }
      } else if (this.withinDomain(uri, constraint)) {
         return true;
      }

      return false;
   }

   private static String extractHostFromURL(String url) {
      if ((url = url.substring(url.indexOf(58) + 1)).indexOf("//") != -1) {
         url = url.substring(url.indexOf("//") + 2);
      }

      if (url.lastIndexOf(58) != -1) {
         url = url.substring(0, url.lastIndexOf(58));
      }

      String var10000 = url.substring(url.indexOf(58) + 1);
      if ((url = var10000.substring(var10000.indexOf(64) + 1)).indexOf(47) != -1) {
         url = url.substring(0, url.indexOf(47));
      }

      return url;
   }

   private String extractNameAsString(GeneralName name) {
      return DERIA5String.getInstance(name.getName()).getString();
   }

   private static byte[] max(byte[] ip1, byte[] ip2) {
      for(int var2 = 0; var2 < ip1.length; ++var2) {
         if ((ip1[var2] & '\uffff') > (ip2[var2] & '\uffff')) {
            return ip1;
         }
      }

      return ip2;
   }

   private static byte[] min(byte[] ip1, byte[] ip2) {
      for(int var2 = 0; var2 < ip1.length; ++var2) {
         if ((ip1[var2] & '\uffff') < (ip2[var2] & '\uffff')) {
            return ip1;
         }
      }

      return ip2;
   }

   private static int compareTo(byte[] ip1, byte[] ip2) {
      if (Arrays.areEqual(ip1, ip2)) {
         return 0;
      } else {
         return Arrays.areEqual(max(ip1, ip2), ip1) ? 1 : -1;
      }
   }

   private static byte[] or(byte[] ip1, byte[] ip2) {
      byte[] var2 = new byte[ip1.length];

      for(int var3 = 0; var3 < ip1.length; ++var3) {
         var2[var3] = (byte)(ip1[var3] | ip2[var3]);
      }

      return var2;
   }

   private int hashCollection(Collection coll) {
      if (coll == null) {
         return 0;
      } else {
         int var3 = 0;
         Iterator coll1 = coll.iterator();

         while(coll1.hasNext()) {
            Object var2;
            if ((var2 = coll1.next()) instanceof byte[]) {
               var3 += Arrays.hashCode((byte[])var2);
            } else {
               var3 += var2.hashCode();
            }
         }

         return var3;
      }
   }

   private boolean collectionsAreEqual(Collection coll1, Collection coll2) {
      if (coll1 == coll2) {
         return true;
      } else if (coll1 != null && coll2 != null) {
         if (coll1.size() != coll2.size()) {
            return false;
         } else {
            Iterator coll11 = coll1.iterator();

            boolean var5;
            do {
               if (!coll11.hasNext()) {
                  return true;
               }

               Object var3 = coll11.next();
               Iterator var4 = coll2.iterator();
               var5 = false;

               while(var4.hasNext()) {
                  if (this.equals(var3, var4.next())) {
                     var5 = true;
                     break;
                  }
               }
            } while(var5);

            return false;
         }
      } else {
         return false;
      }
   }

   private boolean equals(Object o1, Object o2) {
      if (o1 == o2) {
         return true;
      } else if (o1 != null && o2 != null) {
         return o1 instanceof byte[] && o2 instanceof byte[] ? Arrays.areEqual((byte[])o1, (byte[])o2) : o1.equals(o2);
      } else {
         return false;
      }
   }

   private String stringifyIP(byte[] ip) {
      String var4 = "";

      StringBuilder var10000;
      int var2;
      StringBuilder var3;
      for(var2 = 0; var2 < ip.length / 2; ++var2) {
         var10000 = var3 = new StringBuilder;
         var3.<init>(String.valueOf(var4));
         var4 = var10000.append(Integer.toString(ip[var2] & 255)).append(".").toString();
      }

      var4 = var4.substring(0, var4.length() - 1);
      var4 = var4 + "/";

      for(var2 = ip.length / 2; var2 < ip.length; ++var2) {
         var10000 = var3 = new StringBuilder;
         var3.<init>(String.valueOf(var4));
         var4 = var10000.append(Integer.toString(ip[var2] & 255)).append(".").toString();
      }

      return var4.substring(0, var4.length() - 1);
   }

   private String stringifyIPCollection(Set ips) {
      Set var10000 = ips;
      String ips1 = "" + "[";

      StringBuilder var4;
      for(Iterator var2 = var10000.iterator(); var2.hasNext(); ips1 = var4.append(this.stringifyIP((byte[])var2.next())).append(",").toString()) {
         StringBuilder var3;
         var4 = var3 = new StringBuilder;
         var3.<init>(String.valueOf(ips1));
      }

      if (ips1.length() > 1) {
         ips1 = ips1.substring(0, ips1.length() - 1);
      }

      return ips1 + "]";
   }

   public void checkPermittedDN(ASN1Sequence dns) {
      this.checkPermittedDN(this.permittedSubtreesDN, dns);
   }

   public void checkExcludedDN(ASN1Sequence dns) {
      this.checkExcludedDN(this.excludedSubtreesDN, dns);
   }

   protected Set unionDNS(Set excluded, String dns) {
      if (excluded.isEmpty()) {
         if (dns == null) {
            return excluded;
         } else {
            excluded.add(dns);
            return excluded;
         }
      } else {
         Set var10000 = excluded;
         HashSet excluded1;
         excluded1 = new HashSet.<init>();
         Iterator var3 = var10000.iterator();

         while(var3.hasNext()) {
            String var4;
            if (this.withinDomain(var4 = (String)var3.next(), dns)) {
               excluded1.add(dns);
            } else if (this.withinDomain(dns, var4)) {
               excluded1.add(var4);
            } else {
               excluded1.add(var4);
               excluded1.add(dns);
            }
         }

         return excluded1;
      }
   }

   public void checkPermitted(GeneralName name) {
      PKIXNameConstraintValidator var10000;
      PKIXNameConstraintValidator var10001;
      switch (name.getTagNo()) {
         case 1:
            var10000 = this;
            var10001 = this;
            Set var3 = this.permittedSubtreesEmail;
            var10000.checkPermittedEmail(var3, var10001.extractNameAsString(name));
            break;
         case 2:
            this.checkPermittedDNS(this.permittedSubtreesDNS, DERIA5String.getInstance(name.getName()).getString());
         case 3:
         case 5:
         default:
            break;
         case 4:
            this.checkPermittedDN(ASN1Sequence.getInstance(name.getName().getDERObject()));
            break;
         case 6:
            this.checkPermittedURI(this.permittedSubtreesURI, DERIA5String.getInstance(name.getName()).getString());
            break;
         case 7:
            var10000 = this;
            var10001 = this;
            byte[] var2 = ASN1OctetString.getInstance(name.getName()).getOctets();
            var10000.checkPermittedIP(var10001.permittedSubtreesIP, var2);
      }

   }

   public void checkExcluded(GeneralName name) {
      PKIXNameConstraintValidator var10000;
      PKIXNameConstraintValidator var10001;
      switch (name.getTagNo()) {
         case 1:
            var10000 = this;
            var10001 = this;
            Set var3 = this.excludedSubtreesEmail;
            var10000.checkExcludedEmail(var3, var10001.extractNameAsString(name));
            break;
         case 2:
            this.checkExcludedDNS(this.excludedSubtreesDNS, DERIA5String.getInstance(name.getName()).getString());
         case 3:
         case 5:
         default:
            break;
         case 4:
            this.checkExcludedDN(ASN1Sequence.getInstance(name.getName().getDERObject()));
            break;
         case 6:
            this.checkExcludedURI(this.excludedSubtreesURI, DERIA5String.getInstance(name.getName()).getString());
            break;
         case 7:
            var10000 = this;
            var10001 = this;
            byte[] var2 = ASN1OctetString.getInstance(name.getName()).getOctets();
            var10000.checkExcludedIP(var10001.excludedSubtreesIP, var2);
      }

   }

   public void intersectPermittedSubtree(ASN1Sequence permitted) {
      ASN1Sequence var10000 = permitted;
      HashMap permitted1;
      permitted1 = new HashMap.<init>();

      GeneralSubtree var3;
      Integer var4;
      for(Enumeration var2 = var10000.getObjects(); var2.hasMoreElements(); ((Set)permitted1.get(var4)).add(var3)) {
         var3 = GeneralSubtree.getInstance(var2.nextElement());
         Integer var10001 = var4 = new Integer;
         var10001.<init>(var3.getBase().getTagNo());
         if (permitted1.get(var10001) == null) {
            HashSet var5;
            var5 = new HashSet.<init>();
            permitted1.put(var4, var5);
         }
      }

      Iterator var8 = permitted1.entrySet().iterator();

      while(var8.hasNext()) {
         Map.Entry permitted2;
         switch ((Integer)(permitted2 = (Map.Entry)var8.next()).getKey()) {
            case 1:
               this.permittedSubtreesEmail = this.intersectEmail(this.permittedSubtreesEmail, (Set)permitted2.getValue());
               break;
            case 2:
               this.permittedSubtreesDNS = this.intersectDNS(this.permittedSubtreesDNS, (Set)permitted2.getValue());
            case 3:
            case 5:
            default:
               break;
            case 4:
               this.permittedSubtreesDN = this.intersectDN(this.permittedSubtreesDN, (Set)permitted2.getValue());
               break;
            case 6:
               this.permittedSubtreesURI = this.intersectURI(this.permittedSubtreesURI, (Set)permitted2.getValue());
               break;
            case 7:
               this.permittedSubtreesIP = this.intersectIP(this.permittedSubtreesIP, (Set)permitted2.getValue());
         }
      }

   }

   public void intersectEmptyPermittedSubtree(int nameType) {
      PKIXNameConstraintValidator var10000;
      HashSet var2;
      switch (nameType) {
         case 1:
            var10000 = this;
            var2 = new HashSet.<init>();
            var10000.permittedSubtreesEmail = var2;
            break;
         case 2:
            var10000 = this;
            var2 = new HashSet.<init>();
            var10000.permittedSubtreesDNS = var2;
         case 3:
         case 5:
         default:
            break;
         case 4:
            var10000 = this;
            var2 = new HashSet.<init>();
            var10000.permittedSubtreesDN = var2;
            break;
         case 6:
            var10000 = this;
            var2 = new HashSet.<init>();
            var10000.permittedSubtreesURI = var2;
            break;
         case 7:
            var10000 = this;
            var2 = new HashSet.<init>();
            var10000.permittedSubtreesIP = var2;
      }

   }

   public void addExcludedSubtree(GeneralSubtree subtree) {
      PKIXNameConstraintValidator var10000;
      PKIXNameConstraintValidator var10001;
      PKIXNameConstraintValidator var10002;
      Set var2;
      GeneralName subtree1;
      switch ((subtree1 = subtree.getBase()).getTagNo()) {
         case 1:
            var10000 = this;
            var10001 = this;
            var10002 = this;
            var2 = this.excludedSubtreesEmail;
            var10000.excludedSubtreesEmail = var10001.unionEmail(var2, var10002.extractNameAsString(subtree1));
            break;
         case 2:
            var10000 = this;
            var10001 = this;
            var10002 = this;
            var2 = this.excludedSubtreesDNS;
            var10000.excludedSubtreesDNS = var10001.unionDNS(var2, var10002.extractNameAsString(subtree1));
         case 3:
         case 5:
         default:
            break;
         case 4:
            this.excludedSubtreesDN = this.unionDN(this.excludedSubtreesDN, (ASN1Sequence)subtree1.getName().getDERObject());
            break;
         case 6:
            var10000 = this;
            var10001 = this;
            var10002 = this;
            var2 = this.excludedSubtreesURI;
            var10000.excludedSubtreesURI = var10001.unionURI(var2, var10002.extractNameAsString(subtree1));
            break;
         case 7:
            this.excludedSubtreesIP = this.unionIP(this.excludedSubtreesIP, ASN1OctetString.getInstance(subtree1.getName()).getOctets());
      }

   }

   public int hashCode() {
      return this.hashCollection(this.excludedSubtreesDN) + this.hashCollection(this.excludedSubtreesDNS) + this.hashCollection(this.excludedSubtreesEmail) + this.hashCollection(this.excludedSubtreesIP) + this.hashCollection(this.excludedSubtreesURI) + this.hashCollection(this.permittedSubtreesDN) + this.hashCollection(this.permittedSubtreesDNS) + this.hashCollection(this.permittedSubtreesEmail) + this.hashCollection(this.permittedSubtreesIP) + this.hashCollection(this.permittedSubtreesURI);
   }

   public boolean equals(Object o) {
      if (!(o instanceof PKIXNameConstraintValidator)) {
         return false;
      } else {
         PKIXNameConstraintValidator o1;
         Set var2 = (o1 = (PKIXNameConstraintValidator)o).excludedSubtreesDN;
         if (this.collectionsAreEqual(var2, this.excludedSubtreesDN)) {
            var2 = o1.excludedSubtreesDNS;
            if (this.collectionsAreEqual(var2, this.excludedSubtreesDNS)) {
               var2 = o1.excludedSubtreesEmail;
               if (this.collectionsAreEqual(var2, this.excludedSubtreesEmail)) {
                  var2 = o1.excludedSubtreesIP;
                  if (this.collectionsAreEqual(var2, this.excludedSubtreesIP)) {
                     var2 = o1.excludedSubtreesURI;
                     if (this.collectionsAreEqual(var2, this.excludedSubtreesURI)) {
                        var2 = o1.permittedSubtreesDN;
                        if (this.collectionsAreEqual(var2, this.permittedSubtreesDN)) {
                           var2 = o1.permittedSubtreesDNS;
                           if (this.collectionsAreEqual(var2, this.permittedSubtreesDNS)) {
                              var2 = o1.permittedSubtreesEmail;
                              if (this.collectionsAreEqual(var2, this.permittedSubtreesEmail)) {
                                 var2 = o1.permittedSubtreesIP;
                                 if (this.collectionsAreEqual(var2, this.permittedSubtreesIP)) {
                                    PKIXNameConstraintValidator var10000 = this;
                                    PKIXNameConstraintValidator var10001 = this;
                                    Set var3 = o1.permittedSubtreesURI;
                                    if (var10000.collectionsAreEqual(var3, var10001.permittedSubtreesURI)) {
                                       return true;
                                    }
                                 }
                              }
                           }
                        }
                     }
                  }
               }
            }
         }

         return false;
      }
   }

   public String toString() {
      String var1 = "" + "permitted:\n";
      if (this.permittedSubtreesDN != null) {
         var1 = var1 + "DN:\n";
         var1 = var1 + this.permittedSubtreesDN.toString() + "\n";
      }

      if (this.permittedSubtreesDNS != null) {
         var1 = var1 + "DNS:\n";
         var1 = var1 + this.permittedSubtreesDNS.toString() + "\n";
      }

      if (this.permittedSubtreesEmail != null) {
         var1 = var1 + "Email:\n";
         var1 = var1 + this.permittedSubtreesEmail.toString() + "\n";
      }

      if (this.permittedSubtreesURI != null) {
         var1 = var1 + "URI:\n";
         var1 = var1 + this.permittedSubtreesURI.toString() + "\n";
      }

      StringBuilder var10000;
      StringBuilder var2;
      if (this.permittedSubtreesIP != null) {
         var1 = var1 + "IP:\n";
         var10000 = var2 = new StringBuilder;
         var2.<init>(String.valueOf(var1));
         var1 = var10000.append(this.stringifyIPCollection(this.permittedSubtreesIP)).append("\n").toString();
      }

      var1 = var1 + "excluded:\n";
      if (!this.excludedSubtreesDN.isEmpty()) {
         var1 = var1 + "DN:\n";
         var1 = var1 + this.excludedSubtreesDN.toString() + "\n";
      }

      if (!this.excludedSubtreesDNS.isEmpty()) {
         var1 = var1 + "DNS:\n";
         var1 = var1 + this.excludedSubtreesDNS.toString() + "\n";
      }

      if (!this.excludedSubtreesEmail.isEmpty()) {
         var1 = var1 + "Email:\n";
         var1 = var1 + this.excludedSubtreesEmail.toString() + "\n";
      }

      if (!this.excludedSubtreesURI.isEmpty()) {
         var1 = var1 + "URI:\n";
         var1 = var1 + this.excludedSubtreesURI.toString() + "\n";
      }

      if (!this.excludedSubtreesIP.isEmpty()) {
         var1 = var1 + "IP:\n";
         var10000 = var2 = new StringBuilder;
         var2.<init>(String.valueOf(var1));
         var1 = var10000.append(this.stringifyIPCollection(this.excludedSubtreesIP)).append("\n").toString();
      }

      return var1;
   }
}
